

#ifndef SCHEDULER_EXAMPLE
#define SCHEDULER_EXAMPLE


#include "Scheduler.h"
using namespace fastdelegate;
using Scheduler::FrameLockedQueue;
using Scheduler::FixedLimitQueue;
using Scheduler::MovingAverageQueue;

#include <OISEvents.h>
#include <OISInputManager.h>
#include <OISKeyboard.h>

#include <Ogre.h>
using namespace Ogre;


class SchedulerExample : public Singleton<SchedulerExample>, public OIS::KeyListener
{
	public:
		SchedulerExample();

		// int main level methods
		bool initialise();
		void mainLoop();

		// Ogrebits access
		Camera* getCamera() {return _camera;}
		const Camera* getCamera() const {return _camera;}

		SceneManager* getSceneManager() {return _sceneManager;}
		const SceneManager* getSceneManager() const {return _sceneManager;}

		Root* getRoot() {return _root;}
		const Root* getRoot() const {return _root;}

		// scheduler access
		FrameLockedQueue& getFrameLockedQueue() {return _frameLockedQueue;}
		FixedLimitQueue& getFixedLimitTaskQueue() {return _fixedLimitQueue;}
		MovingAverageQueue& getMovingAverageTaskQueue() {return _movingAverageTaskQueue;}

		// simple keyboard event multicasting
		typedef FastDelegate1<const OIS::KeyEvent&> KeyboardCallback;
		void subscribeKeyboard(const std::string& identifier, KeyboardCallback callback);
		void unsubscribeKeyboard(const std::string& identifier);


	protected: // tasks
		bool renderFrame(unsigned long elapsedMS);
		bool captureInput(unsigned long elapsedMS);
		bool frameRate(unsigned long elapsedMS);

	protected: // Ogre data
		Root* _root;
		SceneManager* _sceneManager;
		Camera* _camera;

	protected: // OIS data
		OIS::InputManager* _inputManager;
		OIS::Keyboard* _keyboard;

	protected: // Scheduler stuffs
		FrameLockedQueue _frameLockedQueue;
		FixedLimitQueue _fixedLimitQueue;
		MovingAverageQueue _movingAverageTaskQueue;

		bool _continueRendering;

	protected: // OIS::KeyListener
		virtual bool keyPressed(const OIS::KeyEvent& arg);
		virtual bool keyReleased(const OIS::KeyEvent& arg);

		// simple keyboard event multicasting
		typedef std::map<std::string, KeyboardCallback> KeyboardSubscriberMap;
		KeyboardSubscriberMap _keyboardSubscribers;
};



#endif