// COPYRIGHT  2005, Paul Bridger. All rights reserved.
// Licensed under the new BSD license: see license.txt for details


#include "PythonScheduler.h"
#include "SimpleTime.h"


#include <boost/python.hpp>


namespace Scheduler
{
	FrameLockedQueue* createFrameLockedQueue()
	{
		return new FrameLockedQueue();
	}

	FixedLimitQueue* createFixedLimitQueue()
	{
		return new FixedLimitQueue();
	}

	MovingAverageQueue* createMovingAverageQueue()
	{
		return new MovingAverageQueue();
	}


	TaskId addTaskFrameLockedQueue(FrameLockedQueue& queue, boost::python::object callable, float priority)
	{
		return queue.addTask(callable, priority);
	}

	TaskId addTaskFixedLimitQueue(FixedLimitQueue& queue, boost::python::object callable, unsigned long periodMs, unsigned long varianceMs)
	{
		return queue.addTask(callable, periodMs, varianceMs);
	}

	TaskId addTaskMovingAverageQueue(MovingAverageQueue& queue, boost::python::object callable, unsigned long periodMs, unsigned long varianceMs)
	{
		return queue.addTask(callable, periodMs, varianceMs);
	}
}


#ifdef _DEBUG
BOOST_PYTHON_MODULE(Scheduler_d)
#else
BOOST_PYTHON_MODULE(Scheduler)
#endif
{
	using namespace boost::python;
	using namespace Scheduler;

	class_<FrameLockedQueue>("FrameLockedQueue", no_init)
		.def("step", &FrameLockedQueue::step)
		.def("addTask", &addTaskFrameLockedQueue)
		.def("removeTask", &FrameLockedQueue::removeTask)
	;


	class_<FixedLimitQueue>("FixedLimitQueue", no_init)
		.def("step", &FixedLimitQueue::step)
		.def("addTask", &addTaskFixedLimitQueue)
		.def("removeTask", &FixedLimitQueue::removeTask)
	;


	class_<MovingAverageQueue, boost::noncopyable>("MovingAverageQueue", no_init)
		.def("step", &MovingAverageQueue::step)
		.def("addTask", &addTaskMovingAverageQueue)
		.def("removeTask", &MovingAverageQueue::removeTask)
	;

	def("initialise", &Time::initialise);
	def("createFrameLockedQueue", &Scheduler::createFrameLockedQueue, return_value_policy<manage_new_object>());
	def("createFixedLimitQueue", &Scheduler::createFixedLimitQueue, return_value_policy<manage_new_object>());
	def("createMovingAverageQueue", &Scheduler::createMovingAverageQueue, return_value_policy<manage_new_object>());
}