// COPYRIGHT  2005, Paul Bridger. All rights reserved.
// Licensed under the new BSD license: see license.txt for details


#ifndef UTILITY_MOVING_AVERAGE
#define UTILITY_MOVING_AVERAGE


#include "NonCopyable.h"

#include <deque>


namespace Utility
{
	/** Store data for and compute a moving average.
	*/
	template <typename T>
	class MovingAverage : NonCopyable
	{
		public:
			MovingAverage(size_t maxElements) :
				_maxElements(maxElements),
				_total(0)
			{}

			void setMaxElements(size_t maxElements)
			{
				_maxElements = maxElements;
				resizeElements();
			}

			/// Add a data point
			void push(const T& element)
			{
				_total += element;
				_elements.push_back(element);
				resizeElements();
			}

			/// Compute the average for currently stored data
			T average() const
			{
				if(_elements.empty()) return 0;
				return _total / _elements.size();
			}

		protected:
			void resizeElements()
			{
				while(_elements.size() >= _maxElements)
				{
					_total -= _elements.front();
					_elements.pop_front();
				}
			}

		protected:
			typedef std::deque<T> Container;
			Container _elements;
			size_t _maxElements;

			T _total;
	};
}


#endif