// COPYRIGHT  2005, Paul Bridger. All rights reserved.
// Licensed under the new BSD license: see license.txt for details


#ifndef SCHEDULER_TASK_QUEUE
#define SCHEDULER_TASK_QUEUE


#ifdef PYTHON_SCHEDULER
#include <boost/python.hpp>
#endif

// changes
// qualified calls to superclass methods

namespace Scheduler
{
	/** Coordinate task scheduling.

	The TaskQueue must be parameterised with a couple of policies determining how the queue will operate.

	The TaskOrderingPolicy handles the logic of deciding which task to run next.
	@see PriorityOrdering, UrgencyOrdering

	The StepSmoothingPolicy decides when the current processing frame should end.
	@see NoSmoothing, FixedLimitSmoothing, MovingAverageSmoothing

	This class uses policy-based design partially as a design experiment, and partially because it allows
	good flexibility and extensibility by the library user.

	*/
	template <typename TaskOrderingPolicy, typename StepSmoothingPolicy>
	class TaskQueue : public TaskOrderingPolicy, public StepSmoothingPolicy
	{
		public:
			void step(unsigned long sinceLastMS)
			{
				StepSmoothingPolicy::frameStart(); //changed

				TaskOrderingPolicy::taskOrderingStep(sinceLastMS); //changed

				// this should be made clearer
				bool frameFinished;
				while(frameFinished = StepSmoothingPolicy::continueFrame()) //changed
				{
					if(TaskOrderingPolicy::runNextTask() == false) break; //changed
				}
				if(frameFinished == false)
				{
					TaskOrderingPolicy::reset(); //changed
				}

				StepSmoothingPolicy::frameEnd(); //changed
			}
	};
}


#endif