// this code is heavily derived from OgreWin32Timer.cpp


#include "SimpleTime.h"

#if defined( __WIN32__ ) || defined( _WIN32 )
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include <cassert>


namespace Time
{
	namespace
	{
		LARGE_INTEGER frequency, startTime;
		DWORD startTick = 0;
	}

	void initialise()
	{
		QueryPerformanceFrequency(&frequency);
		QueryPerformanceCounter(&startTime);
		startTick = GetTickCount();
	}

	unsigned long getMilliseconds()
	{
		// check for initialisation
		assert(startTick != 0);

		LARGE_INTEGER currentTime;
		QueryPerformanceCounter(&currentTime);
		
		// scale by 1000 for milliseconds
		unsigned long newTicks = (unsigned long)(1000 * (currentTime.QuadPart - startTime.QuadPart) / frequency.QuadPart);

		// detect and compensate for performance counter leaps
		// (surprisingly common, see Microsoft KB: Q274323)
	    unsigned long check = GetTickCount() - startTick;
		signed long errorMs = (signed long)(newTicks - check);
		if (errorMs < -100 || errorMs > 100)
		{
			startTime.QuadPart += errorMs * frequency.QuadPart / 1000;
			newTicks = check;
		}

		return newTicks;
	}
}

#else // let's assume that anythin non windows supports the unix gettimeofday() function (Macs do) 

/* Code by Sigfried McWild for *nix compatibility */
#include <sys/time.h>

#include <cassert>

// I'm not quite sure if I'm doign this the right way since I don't really knwo the win32 api and what all those calls above do
// I'll do my best anyway
namespace Time
{
	// don't really liek this stuff with globals lying aroudn but I guess it is the best way to do it in this case
	namespace
	{
		unsigned long startTick = 0;
		timeval t; // so I don't have to allocate them over and over
	}
	
	void initialise()
	{
		gettimeofday(&t, 0);
		startTick = t.tv_sec * 1000000 + t.tv_usec; // ticks are stored in microseconds (potentially)
	}

	unsigned long getMilliseconds()
	{
		// check for init
		assert(startTick != 0);

		gettimeofday(&t, 0);

		// let's not worry about the counter leaps, I'm pretty sure unix does nto suffer from those issues
		return (t.tv_sec * 1000000 + t.tv_usec - startTick) / 1000;
	}
}

#endif

